#pragma once

#include "Window.h"
#include "SharedMemory.h"

#include "wayland-client.h"
#include "xdg-shell-client-protocol.h"

#include <vector>
#include <memory>

class WaylandSurface;
class WaylandBuffer;

class WaylandWindowInterface
{

public:

    WaylandWindowInterface();

    ~WaylandWindowInterface();

    void connect();

    void disconnect();

    void run();

    void addWindow(mt::Window* window);

    void mapWindow(mt::Window* window);

private:
    static void registryHandleGlobalCallback(void *data, struct wl_registry *registry, uint32_t name, const char *interface, uint32_t version);
    static void registryHandleGlobalRemoveCallback(void *data, struct wl_registry *registry, uint32_t name);

    void doXdgPong(uint32_t serial);

    void setCompositor(wl_compositor* compositor);

    void setSharedMemory(wl_shm* shared_memory);

    void setXdgBase(xdg_wm_base* xdg_base);

    wl_display* mDisplay{nullptr};
    wl_compositor* mCompositor{nullptr};
    wl_registry_listener mRegistryListener;

    xdg_wm_base* mXdgBase{nullptr};
    xdg_wm_base_listener mXdgBaseListener;

    std::vector<std::unique_ptr<WaylandSurface> > mSurfaces;
    std::shared_ptr<WaylandBuffer> mBuffer;

};
